/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/** gclc:fr.bigeon.gclc.ConsoleApplication.java
 * Created on: Sep 6, 2014 */
package fr.bigeon.gclc;

import java.util.ArrayList;
import java.util.List;

import fr.bigeon.gclc.command.Command;
import fr.bigeon.gclc.command.HelpExecutor;
import fr.bigeon.gclc.command.SubedCommand;
import fr.bigeon.gclc.prompt.CLIPrompter;
import fr.bigeon.gclc.prompt.CLIPrompterMessages;

/** <p>
 * A {@link ConsoleApplication} is an application that require the user to input
 * commands.
 * <p>
 * A typical use case is the following:
 * 
 * <pre>
 * {@link ConsoleApplication} app = new {@link ConsoleApplication#ConsoleApplication(String, String, String) ConsoleApplication("exit", "welcome", "see you latter")};
 * app.{@link ConsoleApplication#add(String, Command) add}("my_command", new {@link Command MyCommand()});
 * app.start();
 * </pre>
 * <p>
 * That will launch in the console application that will display "welcome",
 * return a line and prompt the user for the command until he inputs the
 * <code>exit</code> command. Then it will print "see you latter" and exit the
 * start method.
 *
 * @author Emmanuel BIGEON */
public class ConsoleApplication {

    /** A space character */
    private static final String SPACE = " "; //$NON-NLS-1$
    /** The welcome message */
    private final String header;
    /** The good bye message */
    private final String footer;
    /** The container of commands */
    private final SubedCommand root = new SubedCommand(
            new UnrecognizedCommand());
    /** The state of this application */
    private boolean running;

    /** @param exit the keyword for the exit command of this application
     * @param welcome the header message to display on launch of this
     *            application
     * @param goodbye the message to display on exit */
    public ConsoleApplication(String exit, String welcome, String goodbye) {
        super();
        this.header = welcome;
        this.footer = goodbye;
        root.add(exit, new ExitCommand(this));
    }

    /** <p>
     * Add an executable command to this application
     * 
     * @param key the command keyword
     * @param cmd the command
     * @return if the command was added */
    public boolean add(String key, Command cmd) {
        return root.add(key, cmd);
    }

    /** Launches the prompting application */
    public void start() {
        if (header != null) System.out.println(header);
        running = true;
        do {
            String cmd = CLIPrompter.prompt();
            List<String> args = new ArrayList<>();
            for (String string : cmd.split(SPACE)) {
                if (!string.isEmpty()) args.add(string);
            }
            if (args.size() > 0) {
                root.get(args.get(0)).execute(args.toArray(new String[0]));
            }
        } while (running);
        if (footer != null) System.out.println(footer);
    }

    /** Exit this running application before next command prompt */
    public void exit() {
        running = false;
    }

    /** Adds help command on the given key
     * 
     * @param cmd the handle for help
     * @return if the help command was added */
    public boolean addHelpCommand(String cmd) {
        return root.add(cmd, new HelpExecutor(root));
    }
}

/** <p>
 * A command to exit a {@link ConsoleApplication}.
 *
 * @author Emmanuel BIGEON */
class ExitCommand implements Command {
    /** The application that will be exited when this command runs */
    private final ConsoleApplication app;
    /** The tip of the exit command */
    private final String EXIT = "exit.tip"; //$NON-NLS-1$

    /** @param app the application to exit */
    public ExitCommand(ConsoleApplication app) {
        super();
        this.app = app;
    }

    @Override
    public String tip() {
        return CLIPrompterMessages.getString(EXIT);
    }

    @Override
    public void help(String[] args) {
        System.out.println("\t" + tip()); //$NON-NLS-1$
    }

    @Override
    public void execute(String[] args) {
        app.exit();
    }
}

/** <p>
 * The error message for unrecognized commands
 *
 * @author Emmanuel BIGEON */
class UnrecognizedCommand implements Command {
    /** The unrecognized command key */
    private static final String UNRECOGNIZED_CMD = "unrecognized.cmd"; //$NON-NLS-1$

    @Override
    public String tip() {
        return null;
    }

    @Override
    public void help(String[] args) {
        // Nothing to do (no help provided as this is one (usually)
    }

    @Override
    public void execute(String[] args) {
        System.out.println(CLIPrompterMessages.getString(UNRECOGNIZED_CMD,
                args[0]));
    }
}
